.bottom-panel {
  /* .window-button-icon height +
     .window-button vertical padding +
     .window-button > StWidget vertical padding) */
  height: 2.25em;
}

.window-list {
  spacing: 2px;
  font-size: 10pt;
}

.window-button {
  padding: 1px;
}

.window-button:first-child:ltr {
  padding-left: 2px;
}

.window-button:last-child:rtl {
  padding-right: 2px;
}

.window-button-box {
  spacing: 4px;
}

.window-button > StWidget,
.window-picker-toggle > StWidget {
  color: #bbb;
  background-color: black;
  border-radius: 4px;
  padding: 3px 6px 1px;
  box-shadow: inset 1px 1px 4px rgba(255,255,255,0.5);
  text-shadow: 1px 1px 4px rgba(0,0,0,0.8);
}

.window-picker-toggle {
  padding: 3px;
}

.window-picker-toggle > StWidet {
  border: 1px solid rgba(255,255,255,0.3);
}

.window-button > StWidget {
  -st-natural-width: 18.75em;
  max-width: 18.75em;
}

.window-button:hover > StWidget,
.window-picker-toggle:hover > StWidget {
  color: white;
  background-color: #1f1f1f;
}

.window-button:active > StWidget,
.window-button:focus > StWidget {
  box-shadow: inset 2px 2px 4px rgba(255,255,255,0.5);
}

.window-button.focused > StWidget,
.window-picker-toggle:checked > StWidget {
  color: white;
  box-shadow: inset 1px 1px 4px rgba(255,255,255,0.7);
}

.window-button.focused:active > StWidget,
.window-picker-toggle:checked:active > StWidget {
  box-shadow: inset 2px 2px 4px rgba(255,255,255,0.7);
}

.window-button.minimized > StWidget {
  color: #666;
  box-shadow: inset -1px -1px 4px rgba(255,255,255,0.5);
}

.window-button.minimized:active > StWidget {
  box-shadow: inset -2px -2px 4px rgba(255,255,255,0.5);
}

.window-button-icon {
  width: 24px;
  height: 24px;
}

.window-list-workspace-indicator .status-label-bin {
  background-color: rgba(200, 200, 200, .3);
  border: 1px solid #cccccc;
  padding: 0 3px;
  margin: 3px 0;
}

.window-list-workspace-indicator .workspaces-box {
  spacing: 3px;
  padding: 3px;
}

.window-list-workspace-indicator .workspace {
  border: 1px solid #cccccc;
  width: 52px;
}

.window-list-workspace-indicator .workspace:first-child:last-child:ltr,
.window-list-workspace-indicator .workspace:first-child:last-child:rtl {
  border-radius: 4px;
}

.window-list-workspace-indicator .workspace:first-child:ltr,
.window-list-workspace-indicator .workspace:last-child:rtl {
  border-radius: 4px 0 0 4px;
}

.window-list-workspace-indicator .workspace:first-child:rtl,
.window-list-workspace-indicator .workspace:last-child:ltr {
  border-radius: 0 4px 4px 0;
}

.window-list-workspace-indicator .workspace.active {
  background-color: rgba(200, 200, 200, .3);
}

.window-list-window-preview {
  background-color: #252525;
  border: 1px solid #ccc;
}

.window-list-window-preview.active {
  background-color: #353535;
  border: 2px solid #ccc;
}

.notification {
  font-weight: normal;
}
