@import url("stylesheet.css");

#panel.bottom-panel {
    border-top-width: 1px;
    border-bottom-width: 0px;
    height: 2.25em ;
    padding: 2px;
  }

  .bottom-panel .window-button > StWidget,
  .bottom-panel .window-picker-toggle > StWidget {
    color: #2e3436;
    background-color: #eee;
    border-radius: 3px;
    padding: 3px 6px 1px;
    box-shadow: none;
    text-shadow: none;
    border: 1px solid rgba(0,0,0,0.2);
  }

  .bottom-panel .window-button > StWidget {
    -st-natural-width: 18.7em;
    max-width: 18.75em;
  }

  .bottom-panel .window-button:hover > StWidget,
  .bottom-panel .window-picker-toggle:hover > StWidget {
    background-color: #f9f9f9;
  }

  .bottom-panel .window-button:active > StWidget,
  .bottom-panel .window-button:focus > StWidget {
    box-shadow: inset 0 1px 3px rgba(0,0,0,0.1);
  }

  .bottom-panel .window-button.focused > StWidget,
  .bottom-panel .window-picker-toggle:checked > StWidget {
    background-color: #ccc;
    box-shadow: inset 0 1px 3px rgba(0,0,0,0.1);
  }

  .bottom-panel .window-button.focused:hover > StWidget {
    background-color: #e9e9e9;
  }

  .bottom-panel .window-button.minimized > StWidget {
    color: #888;
    box-shadow: none;
  }

/* workspace switcher */
.window-list-workspace-indicator .workspace {
  background-color: #ddd;
}

.window-list-workspace-indicator .workspace.active {
  background-color: #ccc;
}

.window-list-window-preview {
  background-color: #ededed;
  border: 1px solid #ccc;
}

.window-list-window-preview.active {
  background-color: #f6f5f4;
  border: 2px solid #888;
}
